/*******************************************************************************
   RoBoard GPIO Simple Tool

     This program is for the purpose of demonstrating RoBoard GPIO function. 
     Execute it without arguments to see how to use.

*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#if defined(linux)
    #define stricmp  strcasecmp
#endif

#ifdef WINCE
    #include <windows.h>
	#define stricmp _stricmp
#endif

#if defined(WINCE) || defined(WIN32)
    #include <roboard_dll.h>  // use the DLL version of RoBoIO lib
    // #include <roboard.h>   // use the static version of RoBoIO lib
#else
    #include <roboard.h>
#endif

#define OUTPUT (0)
#define INPUT  (1)

int rbver = 100;

void help(char* s) {
    int l = strlen(s);

    for (; l>=0; l--) if ((s[l] == '/') || (s[l] == '\\')) break;
    if (l >= 0) s = &s[l+1];

	printf(
	    "Usage                                            \n"
	    "=======                                          \n"
	    "%s [ver] [pin] [-i|-o val]                       \n"
	 	"                                                 \n"
	    "   [ver] (RoBoard version)                       \n"
	    "      RB050   : RoBoard RB-050                   \n"
	    "      RB100   : RoBoard RB-100                   \n"
	    "      RB100RD : RoBoard RB-100RD                 \n"
	    "      RB110   : RoBoard RB-110                   \n"
	    "                                                 \n"
	    "   [pin] :                                       \n"
	    "      S1 ~ S24 (for RB-100/RB-100RD)             \n"
	    "      S1 ~ S16 (for RB-110/RB-050)               \n"
	    "                                                 \n"
	    "   [-i] : read digital input on pin              \n"
	    "                                                 \n"
	    "   [-o val] : output val on pin                  \n"
	    "              val = 0 : logic LOW                \n"
	    "              val = 1 : logic HIGH               \n"
    	"                                                 \n"
    	"(Press Enter key to continue...)                 \n"
	    , s
    );
	getchar();

	printf(
    	"example                                          \n"
    	"=========                                        \n"
    	"%s RB100 S1 -o 1                                 \n"
    	"    output logic HIGH on PWM/GPIO pin S1.        \n"
    	"                                                 \n"
    	"%s RB110 S4 -o 0                                 \n"
    	"    output logic LOW on PWM/GPIO pin S4.         \n"
    	"                                                 \n"
    	"%s RB050 S6 -i                                   \n"
    	"    read digital input on PWM/GPIO pin S6.       \n"
    	"                                                 \n"
    	, s, s, s
    );

}

int GetChannel(char* str) {
	int val, maxch;

    if ((str[0] == 'S') || (str[0] == 's'))
    {

        val = atoi(&str[1]);
        if ((rbver == 100) || (rbver == 102) || (rbver == 103)) maxch = 24; else if (rbver == 101) maxch = 32; else maxch = 16;
        if ((val >= 1) && (val <= maxch)) return val-1;
	}

    printf("ERR: wrong PWM/GPIO pin!!!\n\n");
    exit(1);
}

int GetGPIODir(char* str) {
	if(stricmp("-o", str) == 0)
		return OUTPUT;
	else
	if(stricmp("-i", str) == 0)
		return INPUT;

	printf("ERR: wrong third parameter!!!\n\n");
	exit(1);
}

int GetOutVal(char* str) {
	if(strcmp("0", str) == 0)
        return 0;
    else
	if(strcmp("1", str) == 0)
        return 1;

	printf("ERR: wrong output val!!!\n\n");
	exit(1);
}

void SetRBVer(int argc, char* argv[]) {
    if (stricmp(argv[1], "RB050") == 0)
    {
        roboio_SetRBVer(RB_050); rbver = 50; return;
    }
    else
    if (stricmp(argv[1], "RB100") == 0)
    {
        roboio_SetRBVer(RB_100); rbver = 100; return;
    }
    else
    if (stricmp(argv[1], "RB100b1") == 0)
    {
        roboio_SetRBVer(RB_100b1); rbver = 101; return;
    }
    else
    if (stricmp(argv[1], "RB100b2") == 0)
    {
        roboio_SetRBVer(RB_100b2); rbver = 102; return;
    }
    else
    if (stricmp(argv[1], "RB100RD") == 0)
    {
        roboio_SetRBVer(RB_100RD); rbver = 103; return;
    }
    else
    if (stricmp(argv[1], "RB110") == 0)
    {
        roboio_SetRBVer(RB_110); rbver = 110; return;
    }

	printf("ERR: unknown RoBoard version!!!\n\n");
    exit(1);
}

#ifdef WINCE
int _tmain(int argc, _TCHAR* _argv[])
{
	char  argvbuf[50][100] = {'\0'};
	char* argv[50];

	if (argc > 50) argc = 50;
	for (int _argc=0; _argc<argc; _argc++)
	{
		argv[_argc] = &(argvbuf[_argc][0]);
		wcstombs(argv[_argc], _argv[_argc], 100);
	}
#else
int main(int argc, char* argv[])
{
#endif

	int channel, dir, val;

	if(argc == 1)
	{
		help(argv[0]);
		return 0;
	}

	if(argc<4 || argc>5)
	{
		printf("ERR: wrong number of parameters!!!\n\n");
		return 1;
	}
	
	SetRBVer(argc, argv);
	
	channel = GetChannel(argv[2]);
    dir     = GetGPIODir(argv[3]);
    if (dir == OUTPUT) val = GetOutVal(argv[4]);


	// RCSERVO_USENOPIN means that all PWM/GPIO pins are treated as GPIO mode.
    if (rcservo_Init(RCSERVO_USENOPIN) == false)
	{
        printf("ERR: fail to initialize RC Servo lib (%s)!\n", roboio_GetErrMsg());
		return 1;
	}

	if (dir == INPUT)
	{
		printf("The value on PWM/GPIO pin S%d = %s\n", channel+1, (rcservo_InPin(channel)==0)?"LOW":"HIGH");
	}
	else
	{
		printf("Output logic %s on PWM/GPIO pin S%d\n", (val==0)?"LOW":"HIGH", channel+1);
		rcservo_OutPin(channel, val);
	}

	rcservo_Close();
	return 0;
}

