/*******************************************************************************
   RoBoard PWM Simple Tool

     This program is for the purpose of demonstrating RoBoard PWM function. 
     Execute it without arguments to see how to use.

*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#if defined(linux)
    #define stricmp  strcasecmp
#endif

#ifdef WINCE
    #include <windows.h>
	#define stricmp _stricmp
#endif

#if defined(WINCE) || defined(WIN32)
    #include <roboard_dll.h>  // use the DLL version of RoBoIO lib
    // #include <roboard.h>   // use the static version of RoBoIO lib
#else
    #include <roboard.h>
#endif

int rbver = 100;

void help(char* s) {
    int l = strlen(s);

    for (; l>=0; l--) if ((s[l] == '/') || (s[l] == '\\')) break;
    if (l >= 0) s = &s[l+1];

	printf(
	    "Usage                                            \n"
	    "=======                                          \n"
        "%s [ver] [pin] [period] [duty] [count]           \n"
	 	"                                                 \n"
	    "   [ver] (RoBoard version)                       \n"
	    "      RB050   : RoBoard RB-050                   \n"
	    "      RB100   : RoBoard RB-100                   \n"
	    "      RB100RD : RoBoard RB-100RD                 \n"
	    "      RB110   : RoBoard RB-110                   \n"
	    "                                                 \n"
	    "   [pin] :                                       \n"
	    "      S1 ~ S24 (for RB-100/RB-100RD)             \n"
	    "      S1 ~ S16 (for RB-110/RB-050)               \n"
        "                                                 \n"
        "   [period] (PWM period):                        \n"
        "      set PWM period (in microsecond)            \n"
        "      note: period must > duty                   \n"
        "                                                 \n"
        "   [duty] (PWM duty):                            \n"
        "      set PWM duty (in microsecond)              \n"
        "                                                 \n"
        "   [count] : (PWM pulse count)                   \n"
        "      set the number of PWM pulses to output.    \n"
        "                                                 \n"
        "(Press Enter key to continue...)                 \n"
	    , s
	);
	getchar();

	printf(
        "example                                                    \n"
        "=========                                                  \n"
        "%s RB100 S1 10000 1500 200                                 \n"
        "    send 200 PWM pulses with period 10ms and duty 1500us   \n"
        "    on PWM/GPIO pin S1.                                    \n"
        "                                                           \n"
        , s
	);

}

int GetChannel(char* str) {
	int val, maxch;

    if ((str[0] == 'S') || (str[0] == 's'))
    {

        val = atoi(&str[1]);
        if ((rbver == 100) || (rbver == 102) || (rbver == 103)) maxch = 24; else if (rbver == 101) maxch = 32; else maxch = 16;
        if ((val >= 1) && (val <= maxch)) return val-1;
	}

    printf("ERR: wrong PWM/GPIO pin!!!\n\n");
    exit(1);
}

unsigned long GetPeriod(char* str) {
	long period = atol(str);

	if(period <= 0L)
	{
		printf("ERR: period <= 0 !!!\n\n");
		exit(1);
	}

	return (unsigned long) period;
}

unsigned long GetDuty(char* str, unsigned long period) {
	long duty = atol(str);

	if(duty <= 0L)
	{
		printf("ERR: duty <= 0 !!!\n\n");
		exit(1);
	}

	if((unsigned long)duty >= period)
	{
		printf("ERR: duty >= period !!!\n\n");
		exit(1);
	}

	return (unsigned long)duty;
}

unsigned long GetCount(char* str) {
	long count = atol(str);

	if(count <= 0L)
	{
		printf("ERR: count <= 0 !!!\n\n");
		exit(1);
	}

	return (unsigned long)count;
}

void SetRBVer(int argc, char* argv[]) {
    if (stricmp(argv[1], "RB050") == 0)
    {
        roboio_SetRBVer(RB_050); rbver = 50; return;
    }
    else
    if (stricmp(argv[1], "RB100") == 0)
    {
        roboio_SetRBVer(RB_100); rbver = 100; return;
    }
    else
    if (stricmp(argv[1], "RB100b1") == 0)
    {
        roboio_SetRBVer(RB_100b1); rbver = 101; return;
    }
    else
    if (stricmp(argv[1], "RB100b2") == 0)
    {
        roboio_SetRBVer(RB_100b2); rbver = 102; return;
    }
    else
    if (stricmp(argv[1], "RB100RD") == 0)
    {
        roboio_SetRBVer(RB_100RD); rbver = 103; return;
    }
    else
    if (stricmp(argv[1], "RB110") == 0)
    {
        roboio_SetRBVer(RB_110); rbver = 110; return;
    }

	printf("ERR: unknown RoBoard version!!!\n\n");
    exit(1);
}

#ifdef WINCE
int _tmain(int argc, _TCHAR* _argv[])
{
	char  argvbuf[50][100] = {'\0'};
	char* argv[50];

	if (argc > 50) argc = 50;
	for (int _argc=0; _argc<argc; _argc++)
	{
		argv[_argc] = &(argvbuf[_argc][0]);
		wcstombs(argv[_argc], _argv[_argc], 100);
	}
#else
int main(int argc, char* argv[])
{
#endif

	int channel;
	unsigned long usedchannel;
	unsigned long period, duty, count, lastcount;

	if(argc == 1)
	{
		help(argv[0]);
		return 0;
	}

	if (argc != 6)
	{
		printf("ERR: wrong parameter number!!!\n\n");
		return 1;
	}

	SetRBVer(argc, argv);

	channel = GetChannel(argv[2]);
	period  = GetPeriod(argv[3]);
	duty    = GetDuty(argv[4], period);
    count   = GetCount(argv[5]);

	usedchannel = 1L << channel;

	// Initialize PWM interface
    if (rcservo_Init(usedchannel) == false)
	{
        printf("ERR: fail to initialize RC Servo lib (%s)!\n", roboio_GetErrMsg());
		return 1;
	}
	
	rcservo_EnterPWMMode();

	printf("Send %lu PWM pluses with period %luus duty %luus on PWM/GPIO pin S%d..", 
           count, period, duty, channel+1);

    rcservo_SendPWM(channel, period, duty, count);
    for (lastcount = count; (count = rcservo_CheckPWM(channel)) != 0L; )
    {
        if ((lastcount - count) >= 50L) { printf("."); lastcount = count; }
    }
    printf("completed\n");

	rcservo_Close();
	return 0;
}

